using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using Core.LJElements;

namespace Core
{
	/// <summary>
	///    
	/// </summary>
	class CommentsParser
	{
		Regex _comment_info_regexp;

		Regex _comment_regexp;
		Regex _comment_user_regexp;
		Regex _comment_userpic_regexp;
		Regex _comment_parent_regexp;
		Regex _comment_message_regexp;
		Regex _comment_date_regexp;

		public CommentsParser ()
		{
			//------   ------
			//    
			string info_regexp = "<div class=\"b-tree-twig b-tree-twig-(?<indent>\\d*)\".*?data-tid=\"t(?<id>\\d*)\">";

			_comment_info_regexp = new Regex (info_regexp,
				RegexOptions.IgnoreCase | RegexOptions.Multiline |
				RegexOptions.Compiled | RegexOptions.Singleline);


			//  
			//string comments_full_regexp = "data-tid=\"t(?<id>\\d*?)\">.*?<!-- Leaf starts here -->(?<comment>.*?)<!-- Leaf ends here -->\\s*</div>";

			string comments_full_regexp = "data-tid=\"t(?<id>\\d*?)\">(?<comment>.*?)\\s*?</div> </div>  </div>";

			_comment_regexp = new Regex (comments_full_regexp,
				RegexOptions.IgnoreCase | RegexOptions.Multiline |
				RegexOptions.Compiled | RegexOptions.Singleline);


			//  
			string user_regexp = "<div class=\"b-leaf.*?data-displayname=\"(?<user>[\\w\\n-_]*?)\"";

			_comment_user_regexp = new Regex (user_regexp,
				RegexOptions.IgnoreCase | RegexOptions.Multiline |
				RegexOptions.Compiled | RegexOptions.Singleline);


			// 
			string comment_userpic_regexp = "<span class=\"b-leaf-userpic-inner\">\\s*<img src=\"(?<userpic>http://.*?)\"";


			_comment_userpic_regexp = new Regex (comment_userpic_regexp,
				RegexOptions.IgnoreCase | RegexOptions.Multiline |
				RegexOptions.Compiled | RegexOptions.Singleline);


			// 
			string parent_regexp = "<li class=\"b-leaf-actions-item b-leaf-actions-permalink\">.*?thread=(?<parent>\\d*).*?</a>\\s*</li>";

			_comment_parent_regexp = new Regex (parent_regexp,
				RegexOptions.IgnoreCase | RegexOptions.Multiline |
				RegexOptions.Compiled | RegexOptions.Singleline);


			// 
			string message_regexp = "<div class=\"b-leaf-article\">\\s*(?<message>.*?)\\s*</div>\\s*<div class=\"b-leaf-footer\">";

			_comment_message_regexp = new Regex (message_regexp,
				RegexOptions.IgnoreCase | RegexOptions.Multiline |
				RegexOptions.Compiled | RegexOptions.Singleline);


			//  
			string date_regexp = "<span class=\"b-leaf-createdtime\">(?<date>.*?)</span>";

			_comment_date_regexp = new Regex (date_regexp,
				RegexOptions.IgnoreCase | RegexOptions.Multiline |
				RegexOptions.Compiled | RegexOptions.Singleline);
		}


		/// <summary>
		///    
		/// </summary>
		/// <param name="url">    ...1548548.html  </param>
		/// <param name="flathtml">    </param>
		/// <returns> </returns>
		public List<Comment> GetCommentsFromPage (string url, string flathtml)
		{
			List<Comment> pageComments = new List<Comment> ();

			MatchCollection comments_match = _comment_regexp.Matches (flathtml);

			foreach (Match comment_match in comments_match)
			{
				Comment comment = ParseComment (comment_match.Groups["comment"].Value,
					int.Parse (comment_match.Groups["id"].Value), url);

				pageComments.Add (comment);
			}

			return pageComments;
		}


		/// <summary>
		///      
		/// </summary>
		/// <param name="html_page">HTML- </param>
		/// <returns></returns>
		public List<CommentPosition> GetCommentsPositions (string html_page)
		{
			MatchCollection comments_full_match = _comment_info_regexp.Matches (html_page);
			List<CommentPosition> singlePageCommens = new List<CommentPosition> ();

			foreach (Match comment_match in comments_full_match)
			{
				CommentPosition comment =
					new CommentPosition (int.Parse (comment_match.Groups["id"].Value),
					int.Parse (comment_match.Groups["indent"].Value) - 1);

				singlePageCommens.Add (comment);
			}
			return singlePageCommens;
		}



		/// <summary>
		///      html
		/// </summary>
		/// <param name="html"></param>
		/// <returns></returns>
		private Comment ParseComment (string html, int id, string posturl)
		{
			//  
			string username = GetUserName (html);

			// 
			string userpic = GetUserpic (html);

			// 
			int parent = GetParentId (html);

			// 
			string message = GetMessage (html);

			// 
			string date = GetDate (html);

			string url = string.Format ("{0}?thread={1}#t{1}", posturl, id);

			Comment comment = new Comment ();
			comment.Id = id;
			comment.ParentId = parent;
			comment.MainMessage = message;
			comment.Url = url;
			comment.Info.Date = date;

			UserInfo user = new UserInfo ();
			user.UserName = username;
			user.UserpicLink = userpic;
			comment.Info.UserInformation = user;

			return comment;
		}

		private string GetDate (string html)
		{
			string date = "";
			MatchCollection date_match = _comment_date_regexp.Matches (html);
			if (date_match.Count != 0)
			{
				date = date_match[0].Groups["date"].Value;
			}
			return date;
		}

		private string GetMessage (string html)
		{
			string message = "";

			MatchCollection message_match = _comment_message_regexp.Matches (html);
			if (message_match.Count != 0)
			{
				message = message_match[0].Groups["message"].Value;
			}

			return message;
		}

		private int GetParentId (string html)
		{
			MatchCollection parent_match = _comment_parent_regexp.Matches (html);
			int parent = 0;

			if (parent_match.Count != 0)
			{
				parent = int.Parse (parent_match[0].Groups["parent"].Value);
			}
			return parent;
		}

		private string GetUserpic (string html)
		{
			MatchCollection userpic_match = _comment_userpic_regexp.Matches (html);
			string userpic = "";

			if (userpic_match.Count != 0)
			{
				userpic = userpic_match[0].Groups["userpic"].Value;
			}
			return userpic;
		}

		private string GetUserName (string html)
		{
			MatchCollection user_match = _comment_user_regexp.Matches (html);

			string username = "";
			if (user_match.Count != 0)
			{
				username = user_match[0].Groups["user"].Value;
			}
			return username;
		}
	}
}
