﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.IO;
using System.Windows.Forms;
using System.Drawing;

namespace Core
{
	[Serializable]
	public class Config
	{
		private static object _lock = new object ();

		static string _fileName = "config.xml";

		private static Config _inst = null;

		private Config()
		{
			_TemplateDir = Path.Combine (Path.GetDirectoryName (Application.ExecutablePath),
				"templates\\lj\\");
		}

	
		/// <summary>
		/// Размер главной формы
		/// </summary>
		Size _mainFormSize = new Size (900, 700);

		public Size MainFormSize
		{
			get
			{
				return _mainFormSize;
			}

			set
			{
				_mainFormSize = value;
			}
		}

		Point _mainFormLocation = new Point (50, 50);
		public Point MainFormLocation
		{
			get
			{
				return _mainFormLocation;
			}

			set
			{
				_mainFormLocation = value;
			}
		}

		[XmlIgnore]
		public static Config Instance
		{
			get
			{
				lock (_lock)
				{
					if (_inst == null)
					{
						string fullPath = Path.Combine (Path.GetDirectoryName (Application.ExecutablePath), _fileName);

						try
						{
							//Пытаемся загрузить файл с диска и десериализовать его
							using (FileStream fs =	new FileStream (fullPath, FileMode.Open))
							{
								XmlSerializer xs = new XmlSerializer (typeof (Config));
								_inst = (Config)xs.Deserialize (fs);
							}
						}
						catch (Exception)
						{
							//Если не удалось десериализовать то просто создаем новый экземпляр
							_inst = new Config ();
						}
					}
				}

				return _inst;
			}
		}

		public void Save()
		{
			string fullPath = Path.Combine (Path.GetDirectoryName (Application.ExecutablePath), 
				_fileName);

			using (FileStream fs = new FileStream (fullPath, FileMode.Create))
			{
				XmlSerializer xs = new XmlSerializer (typeof (Config));
				xs.Serialize (fs, _inst);
			}
		}

		#region Прокси

		/// <summary>
		/// Использовать прокси?
		/// </summary>
		bool _useProxy = false;

		public bool UseProxy
		{
			get
			{
				return _useProxy;
			}

			set
			{
				_useProxy = value;
			}
		}

		/// <summary>
		/// Адрес прокси
		/// </summary>
		string _proxyAddress = "";

		public string ProxyAddress
		{
			get
			{
				return _proxyAddress;
			}

			set
			{
				_proxyAddress = value;
			}
		}

		/// <summary>
		/// Порт прокси
		/// </summary>
		int _proxyPort = 80;

		public int ProxyPort
		{
			get
			{
				return _proxyPort;
			}

			set
			{
				_proxyPort = value;
			}
		}

		#endregion

		/// <summary>
		/// Путь к текущему скину
		/// </summary>
		string _TemplateDir;

		public string TemplateDir
		{
			get	
			{
				return _TemplateDir;
			}

			set
			{
				_TemplateDir = value;
			}
		}

		
		#region Автообновление

		/// <summary>
		/// Использовать автообновление?
		/// </summary>
		bool _autoUpdate = false;

		public bool AutoUpdate
		{
			get
			{
				return _autoUpdate;
			}

			set
			{
				_autoUpdate = value;
			}
		}


		/// <summary>
		/// Интерал обновления в минутах
		/// </summary>
		int _autoUpdateInterval = 20;

		public int AutoUpdateInterval
		{
			get
			{
				return _autoUpdateInterval;
			}

			set
			{
				_autoUpdateInterval = value;
			}
		}

		#endregion

		#region Главное окно

		int _leftPanelWidth = 250;

		public int LeftPanelWidth
		{
			get { return _leftPanelWidth; }
			set { _leftPanelWidth = value; }
		}

		#endregion

		#region Показывать все комментарии
		/// <summary>
		/// Показывать все комментарии, а не толкьо новые и отмеченные
		/// </summary>
		bool _newOnlyComments = false;

		public event EventHandler<EventArgs> NewOnlyCommentsChange;

		public bool NewOnlyComments
		{
			get { return _newOnlyComments; }
			set 
			{
				if (value != _newOnlyComments)
				{
					_newOnlyComments = value;
					OnShowAllCommentsChange (new EventArgs());
				}
			}
		}

		protected virtual void OnShowAllCommentsChange (EventArgs args)
		{
			if (NewOnlyCommentsChange != null)
			{
				NewOnlyCommentsChange (this, args);
			}
		}

		#endregion

		/// <summary>
		/// Перескакивать на новый комментарий при открытии поста
		/// </summary>
		private bool _gotoNewComment = false;

		public bool GotoNewComment
		{
			get { return _gotoNewComment; }
			set { _gotoNewComment = value; }
		}

		/// <summary>
		/// Показывать в логе время событий
		/// </summary>
		private bool _showLogTime = true;

		public bool ShowLogTime
		{
			get { return _showLogTime; }
			set { _showLogTime = value; }
		}

		/// <summary>
		/// Делать посты, которые давно не обновлялись необновляемыми
		/// </summary>
		private bool _disableOldPost = false;

		public bool DisableOldPost
		{
			get { return _disableOldPost; }
			set { _disableOldPost = value; }
		}


		/// <summary>
		/// Через сколько дней делать посты необновляемыми
		/// </summary>
		private int _oldPostDays = 7;

		public int OldPostDays
		{
			get { return _oldPostDays; }
			set { _oldPostDays = value; }
		}

		/// <summary>
		/// Какого типа показывать посты
		/// </summary>
		private PostType _showPostType;

		/// <summary>
		/// Какого типа показывать посты
		/// </summary>
		public PostType ShowPostType
		{
			get { return _showPostType; }
			set { _showPostType = value; }
		}


		/// <summary>
		/// Имя пользователя
		/// </summary>
		private string _username = "";

		public string Username
		{
			get { return _username; }
			set { _username = value; }
		}


		/// <summary>
		/// Сохранять пароль?
		/// </summary>
		private bool _savePassword = false;

		public bool SavePassword
		{
			get { return _savePassword; }
			set { _savePassword = value; }
		}


		private string _password = "";

		/// <summary>
		/// Сохраненный пароль, хранится в зашифрованном виде с помощью SimpleAES
		/// </summary>
		public string Password
		{
			get { return _password; }
			set { _password = value; }
		}
	}
}
