﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Core.LJElements
{
	[Serializable]
	public class Document
	{
		public Document ()
		{
		}

		List<Post> _posts = new List<Post> ();

		public void AddPost (Post post)
		{
			//_posts.Add (post);
			_posts.Insert (0, post);
			_currSel = 0;
			OnPostListChange (new EventArgs ());
		}

		public void RemovePost (int index)
		{
			_posts.RemoveAt (index);

			if (_currSel >= _posts.Count)
			{
				_currSel = _posts.Count - 1;
			}

			OnPostListChange (new EventArgs ());
		}

		/// <summary>
		/// Можно ли обновлять?
		/// </summary>
		[NonSerialized]
		bool _canUpdate = true;

		public void BeginEdit ()
		{
			_canUpdate = false;
		}

		public void EndEdit ()
		{
			_canUpdate = true;

			if (_skipPostChange)
			{
				OnPostListChange (new EventArgs ());
				_skipPostChange = false;
			}

			if (_skipSelChange)
			{
				OnSelectionChange (new EventArgs ());
				_skipSelChange = false;
			}
		}

		public void MovePostUp (int index)
		{
			if (index != 0)
			{
				Post post = _posts[index];
				_posts.RemoveAt (index);
				_posts.Insert (index - 1, post);

				if (_currSel == index)
				{
					_currSel--;
				}

				OnPostListChange (new EventArgs ());
			}
		}

		public void MovePostDown (int index)
		{
			if (index != _posts.Count - 1)
			{
				Post post = _posts[index];
				_posts.RemoveAt (index);

				_posts.Insert (index + 1, post);

				if (_currSel == index)
				{
					_currSel++;
				}

				OnPostListChange (new EventArgs ());
			}
		}

		#region Изменение списка постов

		public delegate void PostListChangeEventHandler(Object sender, EventArgs args);

		[field: NonSerializedAttribute ()] 
		public event PostListChangeEventHandler PostListChange;

		[NonSerialized]
		bool _skipPostChange = false;

		public void GeneratePostListChangeEvent ()
		{
			OnPostListChange (new EventArgs ());
		}

		protected virtual void OnPostListChange(EventArgs e)
		{
			if (!_canUpdate)
			{
				_skipPostChange = true;
				return;
			}

			if (PostListChange != null)
			{
				PostListChange (this, e);
			}
		}

		#endregion

		public Post this[int index]
		{
			get
			{
				return _posts[index];
			}

			set
			{
				_posts[index] = value;
				OnPostListChange (new EventArgs ());
			}
		}

		public int Count
		{
			get
			{
				return _posts.Count;
			}
		}

		private int _currSel = -1;

		public int SelectedIndex
		{
			get
			{
				return _currSel;
			}

			set
			{
				if (_currSel != value)
				{
					// Пометим старый выбранный пост как прочитанный
					/*if (_currSel != -1)
					{
						_posts[_currSel].SetThreadRead (true);
					}*/

					_currSel = value;

					OnSelectionChange (new EventArgs ());
				}
			}
		}

		#region Изменение выбранной ссылки

		public delegate void SelectionChangeEventHandler(Object sender, EventArgs args);

		[field: NonSerializedAttribute ()] 
		public event SelectionChangeEventHandler SelectionChange;

		[NonSerialized]
		bool _skipSelChange = false;

		protected virtual void OnSelectionChange(EventArgs e)
		{
			if (!_canUpdate)
			{
				_skipSelChange = true;
				return;
			}

			if (SelectionChange != null)
			{
				SelectionChange (this, e);
			}
		}

		#endregion
	}
}
