﻿using System;
using System.Collections.Generic;
using System.Text;

using Core.Exceptions;

namespace Core.LJElements
{
	/// <summary>
	/// Общие члены для комментариев и сообщений
	/// </summary>
	[Serializable]
	public class MessageBase
	{
		/// <summary>
		/// Инфо 
		/// </summary>
		private CommentInfo _info = new CommentInfo ();
		public CommentInfo Info
		{
			get
			{
				return _info;
			}
			set
			{
				_info = value;
			}
		}

		private string _subj;
		public string Subj
		{
			get
			{
				return _subj;
			}
			set
			{
				_subj = value;
			}
		}

		private string _mainMessage;
		public string MainMessage
		{
			get
			{
				return _mainMessage;
			}
			set
			{
				_mainMessage = value;
			}
		}

		private string _url = "";
		virtual public string Url
		{
			get
			{
				return _url;
			}

			set
			{
				_url = value;
			}
		}

		private List<Comment> _comments = new List<Comment> ();

		public void AddComment (Comment comm)
		{
			comm.Parent = this;
			//comm.Level = this.Level + 1;
			_comments.Add (comm);

			// Посчитаем комментарии
			//_commentsCount += comm.CommentsCount + 1;

			//if (this.Parent != null)
		}

		public Comment[] Comments
		{
			get
			{
				return _comments.ToArray ();
			}
		}


		protected MessageBase _parent = null;

		/// <summary>
		/// Кому предназначено сообщение
		/// </summary>
		public virtual MessageBase Parent
		{
			get
			{
				return _parent;
			}

			set
			{
				throw new RootParentException ();
			}
		}

		/// <summary>
		/// Уровень вложенности. 
		/// 0 - Пост
		/// >0 - комментарии
		/// </summary>
		protected int _level = 0;
		virtual public int Level
		{
			get
			{
				return _level;
			}

			set
			{
				_level = value;
			}
		}

		/// <summary>
		/// Был ли прочитан комментарий
		/// </summary>
		private bool _isRead = false;

		/// <summary>
		/// Был ли прочитан комментарий
		/// </summary>
		public bool IsRead
		{
			get
			{
				return _isRead;
			}
		}

		/// <summary>
		/// Есть ли у поста непрочитанные комментарии
		/// </summary>
		private bool _isNewCommentsParent = false;

		/// <summary>
		/// Есть ли у поста непрочитанные комментарии
		/// </summary>
		public bool IsNewCommentsParent
		{
			get
			{
				return _isNewCommentsParent;
			}
		}

		public void SetThreadRead (bool read)
		{
			_isRead = read;

			for (int i = 0; i < _comments.Count; ++i)
			{
				_comments[i].SetThreadRead (read);
			}

			if (_comments.Count == 0 && _parent != null)
			{
				SetParentRead (!read);
			}
		}

		public void SetParentRead (bool isParentNewComment)
		{
			_isNewCommentsParent = isParentNewComment;

			if (_parent != null)
			{
				_parent.SetParentRead (isParentNewComment);
			}
		}

		/// <summary>
		/// Число комемнтариев
		/// </summary>
		/*private int _commentsCount = 0;

		public int CommentsCount
		{
			get
			{
				return _commentsCount;
			}

			// !!!!! Используется только для сериализации
			set
			{
				_commentsCount = value;
			}
		}*/

		/// <summary>
		/// Дада последнего обновления, когда получен новый комментарий
		/// </summary>
		DateTime _lastUpdate = DateTime.Now;

		public DateTime LastUpdate
		{
			get
			{
				return _lastUpdate;
			}

			set
			{
				_lastUpdate = value;
			}
		}

		public int AllCommentsCount
		{
			get
			{
				int ccount = 0;

				for (int i = 0; i < _comments.Count; ++i)
				{
					ccount += _comments[i].AllCommentsCount;
				}

				return ccount + _comments.Count;
			}
		}

		public int NewCommentsCount
		{
			get
			{
				int nccount = 0;

				for (int i = 0; i < _comments.Count; ++i)
				{
					if (_comments[i].IsNewCommentsParent)
					{
						nccount += _comments[i].NewCommentsCount;

						if (!_comments[i].IsRead)
						{
							nccount++;
						}
					}
				}

				return nccount;
			}
		}
	}
}
