﻿using System;
using System.Text;
using System.Windows.Forms;

namespace Core.Log
{
	public class TextLog: ILog
	{
		TextBox _field;

		public TextLog (TextBox field)
		{
			_field = field;
		}

		#region ILog Members

		public void Write(string text)
		{
			lock (_field)
			{
				//_field.Text = _field.Text + text;

				SafeWrite (_field.Text + text);
			}
		}

		private void SafeWrite (string newstr)
		{
			object[] param = new object[2];
			param[0] = _field;
			param[1] = newstr;

			//_field.BeginInvoke (new SetTextDelegate (DelegateMethod), param);

			if (_field.InvokeRequired)
			{
				_field.BeginInvoke (new SetTextDelegate (DelegateMethod), param);
			}
			else
			{
				DelegateMethod (_field, newstr);
			}
		}

		public void Clear()
		{
			lock (_field)
			{
				//_field.Text = "";

				SafeWrite ("");
			}
		}


		public void WriteLine(string text)
		{
			lock (_field)
			{
				//_field.Text = _field.Text + text + "\r\n";


				string newStr;
				if (text.Length == 0)
				{
					newStr = _field.Text + "\r\n";
				}
				else
				{
					DateTime now = DateTime.Now;
					string formatstr = Config.Instance.ShowLogTime ?
						String.Format ("[{0}:{1}:{2}] {3}",
							now.Hour, now.Minute, now.Second, text) :
						text;

					newStr = _field.Text + formatstr + "\r\n";
				}

				SafeWrite (newStr);
			}
		}

		#endregion

		private delegate void SetTextDelegate(TextBox myControl, string text);

		public void DelegateMethod(TextBox myControl, string text)
		{
			myControl.Text = text;
			myControl.SelectionStart = myControl.Text.Length;
			myControl.SelectionLength = 0;
		}

	}
}
