using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Security;
using System.Net;
using System.Text.RegularExpressions;

using Core.LJElements;
using Core.Log;
using System.Net.Cache;


namespace Core
{
	public class PostLoader
	{
		public PostLoader (string url, ILog log, CookieCollection cookies)
		{
			_cookies = cookies;

			MakePostLoader (url, log);
		}

		string _source = "";
		public string Source
		{
			get
			{
				return _source;
			}
		}

		string _content = "";
		public string Content
		{
			get
			{
				return _content;
			}
		}

		Post _post;
		public Post LoadedPost
		{
			get
			{
				return _post;
			}
		}

		ILog _log;

		CookieCollection _cookies = null;

		private void MakePostLoader (string url, ILog log)
		{
			_log = log;

			string lightUrl = GetLightUrl (url);

			//////////////////////////////////////////////////////

			string html = SendRequest (lightUrl);

			PostParser postParser = new PostParser ();
			_post = postParser.ParsePost (url, html);

			List<CommentPosition> comments_info = GetCommentPositions (lightUrl);
			List<Comment> comments_flat = GetFlatComments (url);
			List<Comment> comments_tree = Flat2TreeComments (comments_flat, comments_info, url);

			foreach (Comment comment in comments_tree)
			{
				_post.AddComment (comment);
			}
		}


		private static string GetLightUrl (string url)
		{
			string lightUrl = url;
			if (lightUrl.EndsWith (".html"))
			{
				lightUrl += "?format=light";
				//lightUrl = url;
			}
			else
			{
				lightUrl += "&format=light";
				//lightUrl = url;
			}
			return lightUrl;
		}


		/// <summary>
		///   
		/// </summary>
		/// <param name="lightUrl"></param>
		/// <returns></returns>
		private List<CommentPosition> GetCommentPositions (string lightUrl)
		{
			CommentsParser commentsParser = new CommentsParser ();
			List<string> allPages = LoadAllPages (lightUrl);

			List<CommentPosition> comments_info = new List<CommentPosition> ();

			foreach (string html_page in allPages)
			{
				comments_info.AddRange (commentsParser.GetCommentsPositions (html_page));
			}
			return comments_info;
		}


		private List<Comment> Flat2TreeComments (List<Comment> flat_comments, 
			List<CommentPosition> commentsinfo, string posturl)
		{
			List<Comment> result = new List<Comment> ();
			Dictionary<int, Comment> comments_dict = new Dictionary<int, Comment> ();

			foreach (Comment comment in flat_comments)
			{
				comments_dict.Add (comment.Id, comment);
			}

			int parentlevel = 0;
			MessageBase parent = null;

			foreach (CommentPosition comment in commentsinfo)
			{
				Comment curr_comment;

				if (comments_dict.ContainsKey (comment.Id))
				{
					curr_comment = comments_dict[comment.Id];
				}
				else
				{
					curr_comment = new Comment ();
					curr_comment.Id = comment.Id;
					curr_comment.Subj = " ";

					string url = string.Format ("{0}?thread={1}#t{2}", posturl, 
						curr_comment.Id, curr_comment.Id);
					curr_comment.Url = url;
				}

				while (parent != null && parentlevel != comment.Level)
				{
					parent = parent.Parent;
					parentlevel--;
				}

				if (parent == null)
				{
					result.Add (curr_comment);
				}
				else
				{
					parent.AddComment (curr_comment);
				}

				parent = curr_comment;
				parentlevel++;
			}

			return result;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="url_flat">    ...1548548.html  </param>
		/// <returns></returns>
		private List<Comment> GetFlatComments (string url)
		{
			string url_flat = url + "?format=light&view=flat";
			//string url_flat = url + "?view=flat";
			List<Comment> flat_comments = new List<Comment> ();

			List<string> pages_flat = LoadAllPages (url_flat);

			for (int i = 0; i < pages_flat.Count; i++)
			{
				string flathtml = pages_flat[i];

				//List<Comment> pageComments = GetCommentsFromPage (url, flathtml);
				CommentsParser commentParser = new CommentsParser ();
				List<Comment> pageComments = commentParser.GetCommentsFromPage (url, flathtml);

				flat_comments.AddRange (pageComments);
			}

			return flat_comments;
		}


		/// <summary>
		///  html   
		/// </summary>
		/// <param name="url"></param>
		/// <returns></returns>
		private List<string> LoadAllPages (string url)
		{
			List<string> pages = new List<string> ();


			//  
			string html = SendRequest (url);

			pages.Add (html);
			int pagecount = GetPageCount (html);

			for (int page_number = 2; page_number <= pagecount; page_number++)
			{
				string url_page = string.Format ("{0}&page={1}", url, page_number);
				string page_html = SendRequest (url_page);
				
				pages.Add (page_html);
			}

			return pages;
		}

		private int GetPageCount (string html)
		{
			int count = 1;

			string regex = "<li class=\"b-pager-page";

			Regex pages_regex = new Regex (regex,
				RegexOptions.IgnoreCase | RegexOptions.Multiline |
				RegexOptions.Compiled | RegexOptions.Singleline);

			MatchCollection pages_match = pages_regex.Matches (html);

			if (pages_match.Count != 0)
			{
				//   2, ..     ,  
				count = pages_match.Count / 2;
			}

			return count;
		}


		private string SendRequest (string url)
		{
			HttpWebRequest req = (HttpWebRequest)WebRequest.Create (url);
			req.Proxy = MainFunctions.GetProxy ();
			req.Method = "GET";
			req.UserAgent = "Mozilla/5.0 (Windows NT 5.1; rv:7.0) Gecko/20100101 Firefox/7.0";
			req.Accept = "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8";
			req.Headers.Add ("Accept-Language", "ru,en-us;q=0.7,en;q=0.3"); 
			req.KeepAlive = true;
			req.Headers.Add ("Pragma", "no-cache");
			req.Headers.Add ("Cache-Control", "no-cache");
			req.Headers.Add ("If-None-Match", "GgZz\"e8af617541536888fb48329e873068f0\"");
			req.AllowAutoRedirect = false;
			req.CookieContainer = new CookieContainer ();
			req.CookieContainer.Add (_cookies);

			string ressource = "";
			using (HttpWebResponse result = (HttpWebResponse)req.GetResponse ())
			{
				using (StreamReader rx = new StreamReader (result.GetResponseStream ()))
				{
					ressource = rx.ReadToEnd ();
				}

				if (result.StatusCode == HttpStatusCode.Found)
				{
					_cookies.Add (result.Cookies);
					ressource = SendRequest (result.Headers["Location"]);
				}
			}

			return ressource;
		}
	}
}
