using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

using Core.LJElements;


namespace Core
{
	class PostParser
	{
		Regex _post_community_regexp;
		Regex _post_username_regexp;
		Regex _post_title_regexp;
		Regex _post_message_title_regexp;
		Regex _post_userpic_regexp;


		public PostParser ()
		{
			//   
			//------   ------
			//    
			string username_regexp = "<td class=\"b-singlepost-author-userinfo\">.*?lj:user='(?<user>[\\w\\n-_]*?)'";

			_post_username_regexp = new Regex (username_regexp,
				RegexOptions.IgnoreCase | RegexOptions.Multiline |
				RegexOptions.Compiled | RegexOptions.Singleline);

			//  
			string community_regexp = "<td class=\"b-singlepost-author-userinfo\">.*?<img src='http://l-stat\\.livejournal\\.com/img/community\\.gif.*?<b>(?<community>.*?)</b>.*?<span class=\"b-singlepost-author-date\">";

			_post_community_regexp = new Regex (community_regexp,
				RegexOptions.IgnoreCase | RegexOptions.Multiline |
				RegexOptions.Compiled | RegexOptions.Singleline);


			//  
			string title_regexp = "<title>[\\w\\n-_]*:\\s*(?<title>.*)</title>";

			_post_title_regexp = new Regex (title_regexp,
				RegexOptions.IgnoreCase | RegexOptions.Multiline |
				RegexOptions.Compiled | RegexOptions.Singleline);


			//    
			string message_regexp_title = "<div class=\"b-singlepost-body\">\\s*(?<message>.*)</div>\\s*<br style='clear: both' />";

			_post_message_title_regexp = new Regex (message_regexp_title,
				RegexOptions.IgnoreCase | RegexOptions.Multiline |
				RegexOptions.Compiled | RegexOptions.Singleline);


			// 
			string userpic_regexp = "<td class=\"b-singlepost-author-userpic\">\\s*<img src='(?<userpic>http://.*?)'";

			_post_userpic_regexp = new Regex (userpic_regexp,
				RegexOptions.IgnoreCase | RegexOptions.Multiline |
				RegexOptions.Compiled | RegexOptions.Singleline);
		}


		public Post ParsePost (string url, string html)
		{
			//   
			string username = GetUserName (html);

			// 	
			string title = GetTitle (html);

			//  
			string community = GetCommunity (html);

			// 
			string message = GetMessage (html);

			// 
			string userpic = GetUserpic (html);

			UserInfo user = new UserInfo ();
			user.UserName = username;
			user.UserpicLink = userpic;

			Post post = new Post ();
			post.Subj = title;
			post.Url = url;
			post.MainMessage = message;
			post.Info.UserInformation = user;
			post.Community.UserName = community;

			return post;
		}

		/// <summary>
		///   
		/// </summary>
		/// <param name="html"></param>
		/// <returns></returns>
		private string GetUserpic (string html)
		{
			MatchCollection userpic_match = _post_userpic_regexp.Matches (html);

			string userpic = "";
			if (userpic_match.Count != 0)
			{
				userpic = userpic_match[0].Groups["userpic"].Value;
			}
			return userpic;
		}


		/// <summary>
		///  
		/// </summary>
		/// <param name="html"></param>
		/// <returns></returns>
		private string GetMessage (string html)
		{
			MatchCollection message_match;
			message_match = _post_message_title_regexp.Matches (html);

			string message = "";

			if (message_match.Count == 0)
			{
				throw new FormatException ();
			}

			message = message_match[0].Groups["message"].Value;
			return message;
		}


		/// <summary>
		///  
		/// </summary>
		/// <param name="html"></param>
		/// <returns></returns>
		private string GetTitle (string html)
		{
			MatchCollection title_match = _post_title_regexp.Matches (html);
			string title = "";
			if (title_match.Count != 0)
			{
				title = title_match[0].Groups["title"].Value;
			}
			return title;
		}


		/// <summary>
		///  
		/// </summary>
		/// <param name="html"></param>
		/// <returns></returns>
		private string GetUserName (string html)
		{
			MatchCollection username_match = _post_username_regexp.Matches (html);

			if (username_match.Count == 0)
			{
				throw new FormatException ();
			}

			string username = username_match[0].Groups["user"].Value;
			return username;
		}


		/// <summary>
		/// ,   
		/// </summary>
		/// <param name="html"></param>
		/// <returns></returns>
		private string GetCommunity (string html)
		{
			string community = "";
			MatchCollection community_match = _post_community_regexp.Matches (html);

			if (community_match.Count != 0)
			{
				community = community_match[0].Groups["community"].Value;
			}
			return community;
		}
	}
}
