﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using Core.LJElements;
using Core;

namespace LJWatcher.Gui
{
	/// <summary>
	/// Класс генерит HTML из шаблонов
	/// </summary>
	public class HtmlTemplate
	{
		string _dir;
		const string _postFileName = "post.htm";
		const string _commentFileName = "comment.htm";
		const string _styleFileName = "style.css";
		const string _templateFileName = "template.htm";

		/// <summary>
		/// Шаблон для поста
		/// </summary>
		string _postTemplate;

		/// <summary>
		/// Шаблон для комментария
		/// </summary>
		string _commentTemplate;

		/// <summary>
		/// Шаблон для всего в целом
		/// </summary>
		string _commonTemplate;

		/// <summary>
		/// Конструктор
		/// </summary>
		/// <remarks>В папке должны быть файлы:
		/// post.ljt с шаблоном для поста
		/// comment.ljt с шаблоном для комментария</remarks>
		/// <param name="dir">Папка с шиблоном</param>
		public HtmlTemplate (string dir)
		{
			_dir = dir.Trim();

			if (_dir.EndsWith ("\\"))
			{
				_dir = _dir.Substring (0, _dir.Length - 1);
			}

			string postFile = Path.Combine (_dir, _postFileName);
			string commentFile = Path.Combine (_dir, _commentFileName);
			string templateFile = Path.Combine (_dir, _templateFileName);

			_postTemplate = LoadTemplate (postFile);
			_commentTemplate = LoadTemplate (commentFile);
			_commonTemplate = LoadTemplate (templateFile);
		}

		public string MakeHtml (Post post, int height)
		{
			// Прочтем стили 
			string style = "";

			string styleFile = Path.Combine (_dir, _styleFileName);

			using (TextReader reader = new StreamReader (styleFile))
			{
				style = reader.ReadToEnd ();
			}

			string postHtml = MakePostHtml (post);
			string comments = "";

			MakeAllCommentsHtml (post, ref comments, 0, post.Url);

			//string fullhtml = postHtml + html;

			StringBuilder sb = new StringBuilder (_commonTemplate);
			//sb.AppendFormat ("<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN'><html><head><META http-equiv='Content-Type' content='text/html; charset=UTF-8'></head><style>{0}</style><body>{1}{2}</body></html>", style, postHtml, comments);

			sb.Replace ("{style}", style);
			sb.Replace ("{post}", postHtml);
			sb.Replace ("{comments}", comments);
			sb.Replace ("{dir}", _dir);
			sb.Replace ("{spaceheight}", height.ToString());

			return sb.ToString();
		}

		private void MakeAllCommentsHtml(MessageBase mess, ref string html, int space, string posturl)
		{
			Comment[] comments = mess.Comments;

			if (mess.Parent != null)
			{
				// Само сообщение не пишем - его отдельно
				Comment comment = (Comment)mess;
				html += MakeSingleCommentHtml (comment, space, posturl);
			}

			for (int i = 0; i < comments.Length; ++i)
			{
				if (!Config.Instance.NewOnlyComments ||
					comments[i].IsNewCommentsParent ||
					!comments[i].IsRead)
				{
					MakeAllCommentsHtml (comments[i], ref html, space + 1, posturl);
				}
			}
		}

		private string LoadTemplate (string fname)
		{
			string format = "";

			using (TextReader reader = new StreamReader (fname, Encoding.GetEncoding (1251)))
			{
				format = reader.ReadToEnd ();
			}

			return format;
		}

		private string MakePostHtml(Post post)
		{
			StringBuilder sb = new StringBuilder ();

			string userpic = post.Info.UserInformation.UserpicLink.Length != 0 ?
				("<img hspace='3' src='" +
				post.Info.UserInformation.UserpicLink + "'/>") : "";

			StringBuilder format = new StringBuilder (_postTemplate);

			format.Replace ("{userpic}", userpic);
			format.Replace ("{subj}", post.Subj);
			format.Replace ("{profile}", post.Info.UserInformation.ProfileLink);
			format.Replace ("{journal}", post.Info.UserInformation.JournalLink);
			format.Replace ("{username}", post.Info.UserInformation.UserName);
			format.Replace ("{message}", post.MainMessage);
			format.Replace ("{source}", post.Url);
			format.Replace ("{upics}", post.Info.UserInformation.UserPicsLink);
			format.Replace ("{calendar}", post.Info.UserInformation.CalendarLink);
			format.Replace ("{friends}", post.Info.UserInformation.FriendsLink);
			format.Replace ("{memories}", post.Info.UserInformation.MemoriesLink);
			format.Replace ("{dir}", _dir);
			format.Replace ("{commname}", post.Community.UserName);
			format.Replace ("{commlink}", post.Community.JournalLink);
			format.Replace ("{lastupdate}", post.LastUpdate.ToString());
			format.Replace ("{commentscount}", post.AllCommentsCount.ToString ());
			format.Replace ("{newcommentscount}", post.NewCommentsCount.ToString ());

			return format.ToString();
		}

		private string MakeSingleCommentHtml(Comment comment, int space, string posturl)
		{
			StringBuilder sb = new StringBuilder ();

			string userpic = comment.Info.UserInformation.UserpicLink.Length != 0 ?
				("<img align='left' hspace='3' src='" +
				comment.Info.UserInformation.UserpicLink + "'/>") : "";

			//string BackColor = comment.IsRead ? "#aaccee" : "#ff0000";
			string commentClass;

			if (comment.IsRead)
			{
				// Старый комментарий
				if (comment.IsNewCommentsParent)
				{
					// Есть новые в подветке
					commentClass = "newcommentparent";
				}
				else
				{
					// Нет новых в подветке
					commentClass = "comment";
				}
			}
			else
			{
				// Новый комментарий
				commentClass = "newcomment";
			}

			StringBuilder format = new StringBuilder (_commentTemplate);

			format.Replace ("{posturl}", posturl);
			format.Replace ("{space}", (25 * space).ToString ());
			format.Replace ("{commentclass}", commentClass);
			format.Replace ("{subj}", comment.Subj);
			format.Replace ("{profile}", comment.Info.UserInformation.ProfileLink);
			format.Replace ("{journal}", comment.Info.UserInformation.JournalLink);
			format.Replace ("{username}", comment.Info.UserInformation.UserName);
			format.Replace ("{message}", comment.MainMessage);
			format.Replace ("{userpic}", userpic);
			format.Replace ("{upics}", comment.Info.UserInformation.UserPicsLink);
			format.Replace ("{calendar}", comment.Info.UserInformation.CalendarLink);
			format.Replace ("{friends}", comment.Info.UserInformation.FriendsLink);
			format.Replace ("{memories}", comment.Info.UserInformation.MemoriesLink);
			format.Replace ("{dir}", _dir);
			format.Replace ("{commentdate}", comment.Info.Date);
			format.Replace ("{link}", comment.Url);
			format.Replace ("{id}", comment.Id.ToString());

			Comment parent = comment.Parent as Comment;
			if (parent != null)
			{
				//format.Replace ("{parentid}", parent.Id == 0 ? 
				//	comment.Id.ToString() : parent.Id.ToString());

				format.Replace ("{parentid}", parent.Id.ToString());
			}
			else
			{
				format.Replace ("{parentid}", "0");
			}

			return format.ToString();
		}

	}
}
