﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Core;
using Core.LJElements;

namespace LJWatcher.Gui
{
	public partial class LJListBox : UserControl
	{
		Document _doc = null;

		// Отображаемые посты
		SelectedPost[] _posts;

		public Document Doc
		{
			get
			{
				return _doc;
			}

			set
			{
				if (_doc != null)
				{
					_doc.PostListChange -= new Document.PostListChangeEventHandler (_doc_PostListChange);
					_doc.SelectionChange -= new Document.SelectionChangeEventHandler (_doc_SelectionChange);
				}

				_doc = value;

				if (value != null)
				{
					_doc.PostListChange += new Document.PostListChangeEventHandler (_doc_PostListChange);
					_doc.SelectionChange += new Document.SelectionChangeEventHandler (_doc_SelectionChange);
				}

				UpdateKeywords ();
				UpdateList ();
			}
		}

		/// <summary>
		/// Получить тип отображаемых постов
		/// </summary>
		/// <returns></returns>
		PostType GetPostType()
		{
			switch (postsTypeShow.SelectedIndex)
			{
				case 1:
					return PostType.Updated;
				case 2:
					return PostType.Unread;
				default:
					return PostType.All;
			}
		}

		void UpdateKeywords ()
		{
			List<string> keywords = KeywordsManager.GetAllKeywords (_doc);

			// Предыдущее выбранное ключевое слово
			string oldKeyword = "";
			if (keywordsList.SelectedIndex >= 1)
			{
				oldKeyword = (string)keywordsList.Items[keywordsList.SelectedIndex];
			}

			keywordsList.Items.Clear ();
			keywordsList.Items.Add ("Все");

			// Новый индекс
			int selIndex = 0;

			for (int i = 0; i < keywords.Count; i++)
			{
				// Пропускаем "пустые" ключевые слова
				if (keywords[i].Length > 0)
				{
					keywordsList.Items.Add (keywords[i]);
					if (oldKeyword == keywords[i])
					{
						selIndex = i;
					}
				}
			}

			keywordsList.SelectedIndex = selIndex;
		}

		void UpdateList()
		{
			if (_doc != null)
			{
				string keyword = "";
				if (keywordsList.SelectedIndex >= 1)
				{
					keyword = (string)keywordsList.Items[keywordsList.SelectedIndex];
				}

				_posts = PostFilter.SelectPosts (_doc, GetPostType (), keyword);

				// Обновим список отображаемых постов
				postsList.Items.Clear ();
				for (int i = 0; i < _posts.Length; ++i)
				{
					postsList.Items.Add ("");

					if (_doc.SelectedIndex == _posts[i].SrcNumber)
					{
						postsList.SelectedIndex = i;
					}
				}

				//listBox.SelectedIndex = _doc.SelectedIndex;
				//UpdateKeywords ();
				Invalidate ();
			}
		}

		void _doc_SelectionChange(object sender, EventArgs args)
		{
			UpdateList ();
		}

		void _doc_PostListChange(object sender, EventArgs args)
		{
			UpdateList ();
			UpdateKeywords ();
		}

		public LJListBox()
		{
			InitializeComponent ();

			_DisabledSelBrush = new SolidBrush (DisabledSelTextColor);
			_DisabledBrush = new SolidBrush (DisabledTextColor);

			_EnabledSelBrush = new SolidBrush (EnabledSelTextColor);
			_EnabledBrush = new SolidBrush (EnabledTextColor);

			_normalFont = new Font (postsList.Font, FontStyle.Regular);
			_updatedFont = new Font (postsList.Font, FontStyle.Bold);

			postsTypeShow.Items.Clear ();
			postsTypeShow.Items.Add ("Все");
			postsTypeShow.Items.Add ("Обновляемые");
			postsTypeShow.Items.Add ("Непрочитанные");

			//postsTypeShow.SelectedIndex = 0;
			switch (Config.Instance.ShowPostType)
			{
				case PostType.All:
					postsTypeShow.SelectedIndex = 0;
					break;

				case PostType.Updated:
					postsTypeShow.SelectedIndex = 1;
					break;

				case PostType.Unread:
					postsTypeShow.SelectedIndex = 2;
					break;
			}

			//UpdateKeywords ();
		}

		private void listBox_SelectedIndexChanged(object sender, EventArgs e)
		{
			_doc.SelectedIndex = _posts[postsList.SelectedIndex].SrcNumber;
		}

		#region Цвета

		Color DisabledSelTextColor = Color.Blue;
		Color DisabledTextColor = Color.Gray;

		Color EnabledSelTextColor = Color.Blue;
		Color EnabledTextColor = Color.Black;

		#endregion

		
		#region Кисти, краски и т.п.

		Pen _linePen = new Pen (Color.Gray);

		SolidBrush _DisabledSelBrush;
		SolidBrush _DisabledBrush;

		SolidBrush _EnabledSelBrush;
		SolidBrush _EnabledBrush;

		Font _normalFont;
		Font _updatedFont;

		#endregion

		private void listBox_DrawItem(object sender, DrawItemEventArgs e)
		{
			e.DrawBackground();

			if (_doc != null)
			{
				int selIndex = _doc.SelectedIndex;
				int currIndex = e.Index;


				if (currIndex != -1)
				{
					//Color textColor;
					Brush brush;

					if (!_posts[currIndex].Post.Disabled)
					{
						//textColor =
						//	(selIndex == currIndex ? EnabledSelTextColor : EnabledTextColor);

						brush = (selIndex == _posts[currIndex].SrcNumber ? 
							_EnabledSelBrush : _EnabledBrush);
					}
					else
					{
						//textColor = 
						//	(selIndex == currIndex ? DisabledSelTextColor : DisabledTextColor);
						brush = (selIndex == _posts[currIndex].SrcNumber ? 
							_DisabledSelBrush : _DisabledBrush);
					}

					StringBuilder sb = new StringBuilder ();

					// Заголовок
					string subj = (_posts[currIndex].Post.Subj == null ||
						_posts[currIndex].Post.Subj.Length == 0) ?
							"---" : _posts[currIndex].Post.Subj;

					string text = sb.AppendFormat ("{0} \n({1}) - {2}/{3}", subj,
						_posts[currIndex].Post.Info.UserInformation.UserName,
						_posts[currIndex].Post.AllCommentsCount,
						_posts[currIndex].Post.NewCommentsCount).ToString ();

					Font font = _posts[currIndex].Post.IsNewCommentsParent ? _updatedFont : _normalFont;

					//using (Font font = new Font (e.Font,
					//	_doc[currIndex].IsNewCommentsParent ? FontStyle.Bold : FontStyle.Regular))
					//using (Brush brush = new SolidBrush (textColor))

					StringFormat sf = new StringFormat ();
					sf.LineAlignment = StringAlignment.Center;
					sf.Alignment = StringAlignment.Center;

					e.Graphics.DrawString (text, font, brush, e.Bounds, sf);						

					// Нарисуем внизу линию
					e.Graphics.DrawLine (_linePen, 0, e.Bounds.Bottom - 1,
						e.Bounds.Width, e.Bounds.Bottom - 1);
				}
			}
		}

		private void infoStripMenuItem_Click(object sender, EventArgs e)
		{
			if (_doc != null && _doc.SelectedIndex != -1)
			{
				PostInfoDialog dlg = new PostInfoDialog (_doc, _doc[_doc.SelectedIndex]);
				dlg.ShowDialog ();
			}
		}

		private void listBox_Click(object sender, EventArgs e)
		{

		}

		private void listBox_MouseDown(object sender, MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Right && postsList.SelectedIndex != -1)
			{
				int index = postsList.IndexFromPoint (e.Location);

				if (index != -1)
				{
					if (index != postsList.SelectedIndex)
					{
						postsList.SelectedIndex = index;
					}

					contextMenu.Show (postsList, e.Location);
				}

			}
		}

		private void removeStripMenuItem_Click(object sender, EventArgs e)
		{
			MainFunctions.RemoveSelectedPost (_doc);
		}

		private void listBox_Resize(object sender, EventArgs e)
		{
			postsList.Invalidate ();
		}

		private void moveUpMenuItem_Click (object sender, EventArgs e)
		{
			if (_doc != null && _doc.SelectedIndex != -1)
			{
				_doc.MovePostUp (_doc.SelectedIndex);
			}
		}

		private void moveDownMenuItem_Click (object sender, EventArgs e)
		{
			if (_doc != null && _doc.SelectedIndex != _doc.Count - 1)
			{
				_doc.MovePostDown (_doc.SelectedIndex);
			}
		}

		public void EnabledCriticActions (bool enabled)
		{
			moveUpMenuItem.Enabled = enabled;
			moveDownMenuItem.Enabled = enabled;
			infoStripMenuItem.Enabled = enabled;
			removeStripMenuItem.Enabled = enabled;
		}

		private void postsTypeShow_SelectedIndexChanged (object sender, EventArgs e)
		{
			Config.Instance.ShowPostType = GetPostType ();
			UpdateList ();
		}

		private void keywordsList_SelectedIndexChanged (object sender, EventArgs e)
		{
			UpdateList ();
		}
	}
}
