﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.IO;

using Core.LJElements;
using Core;

namespace LJWatcher.Gui
{
	public partial class PostThree : UserControl
	{
		public PostThree()
		{
			InitializeComponent ();

			_postPath = Path.Combine (Path.GetDirectoryName (Application.ExecutablePath),
					"temp\\post.html");

			Config.Instance.NewOnlyCommentsChange 
				+= new EventHandler<EventArgs> (Instance_NewOnlyCommentsChange);
		}

		void Instance_NewOnlyCommentsChange (object sender, EventArgs e)
		{
			UpdatePost ();
		}

		Document _doc = null;
		HtmlTemplate _template = null;

		public HtmlTemplate Template
		{
			get
			{
				return _template;
			}

			set
			{
				_template = value;
				UpdatePost ();
			}
		}

		public Document Doc
		{
			get
			{
				return _doc;
			}

			set
			{
				if (value == null)
				{
					return;
				}

				if (_doc != null)
				{
					_doc.PostListChange -= new Document.PostListChangeEventHandler (_doc_PostListChange);
					_doc.SelectionChange -= new Document.SelectionChangeEventHandler (_doc_SelectionChange);
				}

				_doc = value;
				_doc.PostListChange += new Document.PostListChangeEventHandler (_doc_PostListChange);
				_doc.SelectionChange += new Document.SelectionChangeEventHandler (_doc_SelectionChange);

				UpdatePost ();
			}
		}

		void _doc_SelectionChange(object sender, EventArgs args)
		{
			UpdatePost ();
		}

		void _doc_PostListChange(object sender, EventArgs args)
		{
			UpdatePost ();
		}

		// Можно ли обновлять вид
		bool _canUpdate = true;

		/// <summary>
		/// Путь к временному файлу, куда пишем пост.
		/// </summary>
		string _postPath;

		List<int> _newComments;

		/// <summary>
		/// Текущий комментарий, куда перескакиваем
		/// </summary>
		int _currentComment = -1;
		
		public void UpdatePost()
		{
			if (_doc == null || _template == null || _doc.SelectedIndex == -1)
			{
				webBrowser.DocumentText = "";
				return;
			}

			if (_canUpdate)
			{
				lock (_doc)
				{
					_newComments = new List<int> ();

					Post post = _doc[_doc.SelectedIndex];
					_postHtml = _template.MakeHtml (post, this.Height);

					/*using (StreamWriter sw = new StreamWriter (_postPath, false))
					{
						sw.Write (_postHtml);
					}*/

					_currentComment = 0;
					GetNewComments (post);

					if (_newComments.Count != 0 && Config.Instance.GotoNewComment)
					{
						_currentComment = _newComments[0];
					}

					OpenPost ();
				}
			}
		}

		string _postHtml;

		public class MoveToCommentEventArgs: EventArgs
		{
			//public readonly int NewCommentsCount;
			//public readonly int CurrentComment;

			public bool IsFirstNewComment;
			public bool IsLastNewComment;

			public MoveToCommentEventArgs (bool first, bool last)
			{
				IsFirstNewComment = first;
				IsLastNewComment = last;
			}
		}

		protected void OnMoveToComment (MoveToCommentEventArgs args)
		{
			if (MoveToComment != null)
			{
				MoveToComment (this, args);
			}
		}

		public event EventHandler<MoveToCommentEventArgs> MoveToComment;

		int GetCommentIndexById (int id)
		{
			int index = -1;
			for (int i = 0; i < _newComments.Count; i++)
			{
				if (_newComments[i] == id)
				{
					index = i;
					break;
				}
			}

			return index;
		}

		public void NextComment ()
		{
			int newindex = GetCommentIndexById(_currentComment);
			if (newindex == -1 && _newComments.Count != 0)
			{
				_currentComment = _newComments[0];
				GoToComment ();
			}
			else if (newindex < _newComments.Count - 1)
			{
				_currentComment = _newComments[newindex + 1];
				GoToComment ();
			}
		}

		public void PrevComment ()
		{
			int newindex = GetCommentIndexById (_currentComment);

			if (newindex > 0)
			{
				_currentComment = _newComments[newindex - 1];
				GoToComment ();
			}
		}

		private void OpenPost ()
		{
			webBrowser.DocumentText = _postHtml;
		}

		void GetNewComments (MessageBase message)
		{
			for (int i = 0; i < message.Comments.Length; i++)
			{
				Comment comment = message.Comments[i];

				if (!comment.IsRead && comment.Id > 0)
				{
					// Удаленные комментарии не добавляем
					_newComments.Add (comment.Id);
				}

				if (comment.IsNewCommentsParent)
				{
					GetNewComments (comment);
				}
			}
		}

		private void webBrowser_Navigating(object sender, WebBrowserNavigatingEventArgs e)
		{
			string url = e.Url.ToString ();

			/*if (e.TargetFrameName.Length != 0)
			{
				e.Cancel = true;
				return;
			}*/

			if (!url.StartsWith ("about:blank") && 
				!url.StartsWith ("file://") && 
				!url.StartsWith ("#") &&
				e.TargetFrameName.Length == 0)
			{
				// Заменим &amp; на &

				url = url.Replace ("&amp;", "&");

				Process.Start (url);
				e.Cancel = true;
			}
			else if (url.StartsWith ("about:blank"))
			{
				int pos = url.IndexOf ("#t");
				if (pos != -1)
				{
					e.Cancel = true;
					string strid = url.Substring (pos + 2);

					try
					{
						_currentComment = int.Parse (strid);
						OpenPost ();

					}
					catch (FormatException)
					{
						return;
					}
				}
			}
			
		}

		public string Html
		{
			get
			{
				return webBrowser.DocumentText;
			}
		}

		private void webBrowser_PreviewKeyDown (object sender, PreviewKeyDownEventArgs e)
		{
			const int WM_KEYDOWN = 0x0100;

			Message msg = new Message ();
			msg.HWnd = this.Handle;
			msg.Msg = WM_KEYDOWN;
			//msg.WParam = e.KeyValue;

			ProcessCmdKey (ref msg, e.KeyData); 
		}

		protected override bool ProcessCmdKey (ref Message msg, Keys keyData)
		{
			return base.ProcessCmdKey (ref msg, keyData);
		}

		private void webBrowser_DocumentCompleted (object sender, 
			WebBrowserDocumentCompletedEventArgs e)
		{
			GoToComment ();
		}

		private void GoToComment ()
		{
			string anchor = string.Format ("t{0}", _currentComment);

			HtmlDocument document = webBrowser.Document;
			if (document.All.Count != 0)
			{
				HtmlElement anchorElement = document.All[anchor];

				if (anchorElement != null)
				{
					anchorElement.ScrollIntoView (true);
				}
			}

			int index = GetCommentIndexById (_currentComment);

			OnMoveToComment (new MoveToCommentEventArgs (index == 0 || index == -1,
				index == _newComments.Count - 1));
		}
	}
}