﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Net;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using System.Xml.Serialization;
using System.Threading;
using System.Reflection;

using Core;
using Core.LJElements;
using Core.Log;
using LJWatcher.Gui.Dialogs;
using LJWatcher.Gui;
using LJWatcher.Update;
using LJWatcher.Gui.NewCommentsNotify;

namespace LJWatcher
{
	internal partial class MainForm : Form
	{
		Document _doc = new Document();
		ILog _log = null;

		string _linksFilePath;

		void SetStatusText (string text)
		{
			trayIcon.Text = "LJWatcher  " + text;
			currentState.Text = text;
		}

		private void EnabledCriticActions(bool enabled)
		{
			addToolStripMenuItem.Enabled = enabled;
			removeMenuItem.Enabled = enabled;
			updateMenuItem.Enabled = enabled;
			addBtn.Enabled = enabled;
			removeBtn.Enabled = enabled;
			updateBtn.Enabled = enabled;
			UpdateToolStripMenuItem.Enabled = enabled;
			addPostMenuItem.Enabled = enabled;
			savePost.Enabled = enabled;
			singleUpdate.Enabled = enabled;
			updateMenuItem.Enabled = enabled;
			saveBtn.Enabled = enabled;
			singleUpdateBtn.Enabled = enabled;
			postOptionsMenuItem.Enabled = enabled;

			markReadButton.Enabled = enabled;
			markReadMenuItem.Enabled = enabled;

			markNewButton.Enabled = enabled;
			markNewMenuItem.Enabled = enabled;

			postsList.EnabledCriticActions (enabled);

			if (enabled && Config.Instance.AutoUpdate)
			{
				updateTimer.Start ();
			}
			else
			{
				updateTimer.Stop ();
			}
		}

		CookieCollection _cookies = new CookieCollection();

		public MainForm()
		{
			InitializeComponent ();

			_log = new TextLog (log);

			Config.Instance.NewOnlyCommentsChange += new EventHandler<EventArgs> (Instance_ShowAllCommentsChange);

			_linksFilePath = Path.Combine (Path.GetDirectoryName (Application.ExecutablePath),
				"links.lj");		

			string backupFile = Path.Combine (Path.GetDirectoryName (Application.ExecutablePath),
				"links.bak");

			try
			{
				File.Copy (_linksFilePath, backupFile, true);

				using (FileStream fs = File.OpenRead (_linksFilePath))
				{
					BinaryFormatter format = new BinaryFormatter ();
					_doc = (Document)format.Deserialize (fs);
					_doc.EndEdit ();
				}
			}
			catch (FileNotFoundException)
			{

			}
			catch (SerializationException)
			{
			}

			_doc.PostListChange += new Document.PostListChangeEventHandler (_doc_PostListChange);

			// Настройка дерева комментов
			postThree.Doc = _doc;
			postsList.Doc = _doc;

			HtmlTemplate _template = new HtmlTemplate (Config.Instance.TemplateDir);
			postThree.Template = _template;

			// Настройка панелей
			leftPanel.Width = Config.Instance.LeftPanelWidth;

			// Настройка автообновлений
			SetAutoUpdate ();

			updater.CurrentVersion = Assembly.GetExecutingAssembly ().GetName ().Version;

			_log.WriteLine (string.Format ("LJWatcher {0}", 
				Assembly.GetExecutingAssembly ().GetName ().Version));
			_log.WriteLine ("");

			UpdateNewOnlyComments ();
		}

		void Instance_ShowAllCommentsChange (object sender, EventArgs e)
		{
			UpdateNewOnlyComments ();
		}

		private void SetAutoUpdate()
		{
			updateTimer.Interval = Config.Instance.AutoUpdateInterval;

			if (Config.Instance.AutoUpdate)
			{
				updateTimer.Start ();
			}
			else
			{
				updateTimer.Stop ();
			}
		}

		void _doc_PostListChange(object sender, EventArgs args)
		{
			bool newcomment = false;

			// Проверим наличие новых комментариев
			for (int i = 0; i < _doc.Count; ++i )
			{
				if (_doc[i].IsNewCommentsParent)
				{
					newcomment = true;
					break;
				}
			}

			if (newcomment)
			{
				trayIcon.Icon = new Icon (this.GetType (), "newcomment.ico");
			}
			else
			{
				System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager (typeof (MainForm));
				this.trayIcon.Icon = ((System.Drawing.Icon)(resources.GetObject ("trayIcon.Icon")));
			}

			SaveLinks ();
		}

		private void SaveLinks()
		{
			using (FileStream fs = File.Create (_linksFilePath))
			{
				BinaryFormatter format = new BinaryFormatter ();
				format.Serialize (fs, _doc);
			}
		}

		private void addToolStripMenuItem_Click(object sender, EventArgs e)
		{
			AddDialog dlg = new AddDialog (_doc);

			if (dlg.ShowDialog() == DialogResult.OK)
			{
				SetStatusText ("Загрузка новой записи…");

				EnabledCriticActions (false);

				string url = dlg.Url.Trim();
				if (!url.StartsWith ("http://"))
				{
					url = "http://" + url;
				}

				trayIcon.Icon = new Icon (this.GetType (), "trayupdate.ico");

				_log.WriteLine (String.Format ("Загрузка новой записи {0}…", url));
				newPostThread.RunWorkerAsync (
					new NewPostArgs (_log, url, dlg.Keywords));
			}
		}

		private void about_Click(object sender, EventArgs e)
		{
			About2 dlg = new About2 ();
			dlg.ShowDialog ();
		}

		private void newPostthread_DoWork(object sender, DoWorkEventArgs e)
		{
			NewPostArgs args = (NewPostArgs)e.Argument;

			try
			{
				PostLoader loader;

				loader = new PostLoader (args.Url, args.Log, _cookies);

				Post result = loader.LoadedPost;
				result.Keywords = args.Keywords;

				e.Result = result;
			}
			catch (FormatException)
			{
				args.Log.WriteLine ("Ошибка формата страницы " + args.Url);
				e.Cancel = true;
			}
			catch (Exception)
			{
				args.Log.WriteLine ("Ошибка загрузки записи " + args.Url);
				e.Cancel = true;
			}
		}

		class NewPostArgs
		{
			public ILog Log;
			public string Url;
			public List<string> Keywords;

			public NewPostArgs(ILog log, 
				string url, 
				List<string> keywords)
			{
				this.Log = log;
				Url = url;
				Keywords = keywords;
			}
		};

		private void newPostThread_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			System.ComponentModel.ComponentResourceManager resources = 
				new System.ComponentModel.ComponentResourceManager (typeof (MainForm));

			this.trayIcon.Icon = 
				((System.Drawing.Icon)(resources.GetObject ("trayIcon.Icon")));

			if (!e.Cancelled)
			{
				Post post = (Post)e.Result;
				post.Disabled = false;

				post.SetThreadRead (post.Comments.Length == 0);
				post.SetParentRead (post.Comments.Length != 0);

				_doc.BeginEdit ();
				lock (_doc)
				{
					_doc.AddPost (post);
				}
				_doc.EndEdit ();

				_log.WriteLine ("Загрузка новой записи завершена");

				if (post.Comments.Length != 0)
				{
					trayIcon.Icon = new Icon (this.GetType (), "newcomment.ico");

					INotifer notifer = new TrayNotify (trayIcon);
					notifer.AddPost (post);
				}
			}
			else
			{
				_log.WriteLine ("Загрузка новой записи прервана");
			}

			SetStatusText ("");

			EnabledCriticActions (true);
			//trayIcon.Icon = new Icon (this.GetType (), "newcomment.ico");

		}

		private void updatePost_DoWork(object sender, DoWorkEventArgs e)
		{
			UpdateThreadParams param = (UpdateThreadParams)e.Argument;

			ContentUpdater updater = new ContentUpdater (param);
			updater.StartUpdate ();

			e.Result = updater.Result;
			e.Cancel = updatePostThread.CancellationPending && updater.Success;
		}

		private void updatePost_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			if (!e.Cancelled)
			{
				UpdateThreadParams.PostPosition[] postList =
					(UpdateThreadParams.PostPosition[])e.Result;

				lock (_doc)
				{
					_doc.BeginEdit ();
					_doc.SelectedIndex = -1;

					//INotifer notifer = new PopupNotify (popupManager);
					INotifer notifer = new TrayNotify (trayIcon);

					// Новые посты добавим в документ
					for (int i = 0; i < postList.Length; ++i)
					{
						if (postList[i] != null)
						{
							int index = postList[i].index;
							Post post = postList[i].post;

							post.LastUpdate = _doc[index].LastUpdate;
							post.Disabled = _doc[index].Disabled;
							//post.UseCookies = _doc[index].UseCookies;
							post.Subj = _doc[index].Subj;
							post.Keywords = _doc[index].Keywords;

							post.SetThreadRead (true);

							CommentsComparer.MarkNewComments (_doc[index], post);

							if (post.IsNewCommentsParent)
							{
								post.LastUpdate = DateTime.Now;

								//popupManager.AddPopup (new Popup (post));
								notifer.AddPost (post);
							}
							else if (Config.Instance.DisableOldPost &&
								post.LastUpdate.AddDays (Config.Instance.OldPostDays) <
									DateTime.Now)
							{
								post.Disabled = true;

								string text = string.Format ("В записи '{0}' давно не было новых комментариев, поэтому он становится необновляемым",
									post.Subj.Length == 0 ? "---" : post.Subj);
								_log.WriteLine (text);
							}

							_doc[index] = post;
						}

					}	// for (int i = 0; i < postList.Length; ++i)
				}

				_doc.EndEdit ();
				SaveLinks ();

				_log.WriteLine ("Обновление записей завершено");
			}
			else
			{
				_log.WriteLine ("Обновление записей прервано");

				System.ComponentModel.ComponentResourceManager resources =
					new System.ComponentModel.ComponentResourceManager (typeof (MainForm));
				this.trayIcon.Icon =
					((System.Drawing.Icon)(resources.GetObject ("trayIcon.Icon")));
			}

			SetStatusText ("");
			EnabledCriticActions (true);

			//System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager (typeof (MainForm));
			//this.trayIcon.Icon = ((System.Drawing.Icon)(resources.GetObject ("trayIcon.Icon")));
			
		}

		private void updateMenuItem_Click(object sender, EventArgs e)
		{
			UpdateAllPosts ();
		}

		private void UpdateAllPosts()
		{
			//bool isLogin = false;

			UpdateThreadParams param = new UpdateThreadParams (_log, 
				_cookies,
				updatePostThread);

			for (int i = 0; i < _doc.Count; ++i)
			{
				if (!_doc[i].Disabled)
				{
					param.Add (_doc[i], i);
				}
			}

			SetStatusText ("Обновление записей");
			_log.WriteLine ("Обновление записей…");

			EnabledCriticActions (false);
			trayIcon.Icon = new Icon (this.GetType (), "trayupdate.ico");

			updatePostThread.RunWorkerAsync (param);
		}

		private void mainForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			SaveLinks ();

			// Сохраним размер и положение окна
			Config.Instance.MainFormSize = this.Size;
			Config.Instance.MainFormLocation = this.Location;

			Config.Instance.LeftPanelWidth = leftPanel.Width;

			Config.Instance.Save ();
		}

		private void removeMenuItem_Click(object sender, EventArgs e)
		{
			MainFunctions.RemoveSelectedPost (_doc);
		}

		private void exitToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.Close ();
		}


		private void mainForm_Load(object sender, EventArgs e)
		{
			this.Size = Config.Instance.MainFormSize;
			this.Location = Config.Instance.MainFormLocation;
			postsList.Height = (int)(this.Height * 0.7);

			// Запишем хендл окна, чтобы потом можно было бы его открыть из другого приложения

			string RunDataFileName = Path.Combine (Path.GetDirectoryName (Application.ExecutablePath),
				"rundata.dat");
			using (FileStream fs = File.Create (RunDataFileName))
			using (BinaryWriter writer = new BinaryWriter (fs))
			{
				writer.Write ((Int32)this.Handle);
			}

			Login ();
		}


		private void Login ()
		{
			AuthForm dlg = new AuthForm ();

			/*if (dlg.ShowDialog () == DialogResult.OK)
			{
			}
			else
			{
			}*/

			dlg.ShowDialog ();
			_cookies = dlg.Cookies;
		}

		private void timer1_Tick(object sender, EventArgs e)
		{
			htmlUpdateTimer.Enabled = false;
			postThree.UpdatePost ();
		}

		private void trayIcon_MouseClick(object sender, MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
			{
				this.Visible = !this.Visible;
			}
		}

		const int WM_SYSCOMMAND = 0x0112;
		const int SC_MINIMIZE = 0xF020;

		protected override void WndProc(ref Message m)
		{
			if (m.Msg == WM_SYSCOMMAND && ((int)m.WParam) == SC_MINIMIZE)
			{
				Visible = false;
			}
			else
			{
				base.WndProc (ref m);
			}

		}

		private void stopUpdate_Click(object sender, EventArgs e)
		{
			if (updatePostThread.IsBusy)
			{
				if (MessageBox.Show ("Вы действительно хотите прервать обновление", 
				"Прервать обновление?", 
				MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
				{
					updatePostThread.CancelAsync ();
				}
			}
		}

		private void savePost_Click(object sender, EventArgs e)
		{
			if (saveHtmlDialog.ShowDialog() == DialogResult.OK)
			{
				string html = postThree.Html;

				using (TextWriter fp = new StreamWriter(saveHtmlDialog.FileName))
				{
					fp.Write (html);
				}
			}
		}

		private void logToClipboard_Click(object sender, EventArgs e)
		{
			Clipboard.SetText (log.Text);
		}

		private void singleUpdate_Click(object sender, EventArgs e)
		{
			int index = _doc.SelectedIndex;
			if (index == -1)
			{
				MessageBox.Show ("Выберите запись для обновления", "Нет выбранной записи",
					MessageBoxButtons.OK, MessageBoxIcon.Information);
				return;
			}

			//bool isLogin = false;
			UpdateThreadParams param = new UpdateThreadParams (_log, 
				_cookies,
				updatePostThread);

			param.Add (_doc[index], index);

			SetStatusText ("Обновление записей");
			_log.WriteLine ("Обновление записей…");

			EnabledCriticActions (false);
			trayIcon.Icon = new Icon (this.GetType (), "trayupdate.ico");

			updatePostThread.RunWorkerAsync (param);
		}

		private void configToolStripMenuItem_Click(object sender, EventArgs e)
		{
			ConfigDialog dlg = new ConfigDialog ();

			if (dlg.ShowDialog() == DialogResult.OK)
			{
				postThree.Template = dlg.NewTemplate;

				SetAutoUpdate ();
			}
		}

		private void log_TextChanged(object sender, EventArgs e)
		{
			log.Select (log.Text.Length, 0);
			log.ScrollToCaret ();
		}

		private void updateTimer_LongTimerTick(object sender, LongTimer.LongTimerTickEventArgs args)
		{
			_log.WriteLine ("Автообновление");
			UpdateAllPosts ();
		}

		private void postOptionsMenuItem_Click(object sender, EventArgs e)
		{
			if (_doc.SelectedIndex != -1)
			{
				PostInfoDialog dlg = new PostInfoDialog (_doc, _doc[_doc.SelectedIndex]);
				dlg.ShowDialog ();
			}
			else
			{
				MessageBox.Show ("Вы не выбрали запись для просмотра информации", "Выберите запись",
					MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void listExportMenuItem_Click(object sender, EventArgs e)
		{
			if (saveHtmlDialog.ShowDialog() == DialogResult.OK)
			{
				string html = "<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN'><html><head><META http-equiv='Content-Type' content='text/html; charset=UTF-8'></head><body><table>";

				for (int i = 0; i < _doc.Count; ++i)
				{
					string subj = _doc[i].Subj.Length == 0 ? _doc[i].Url : _doc[i].Subj;


					string info = string.Format ("<tr><td><a href='{0}'>{1}</a></td> <td><a href='{2}'>{3}</a></td></tr>",
						_doc[i].Info.UserInformation.ProfileLink,
						_doc[i].Info.UserInformation.UserName,
						_doc[i].Url,
						subj);

					html += info;
				}

				html += "</table></body></html>";

				using (TextWriter fp = new StreamWriter (saveHtmlDialog.FileName))
				{
					fp.Write (html);
				}
			}
		}

		private void markReadButton_Click (object sender, EventArgs e)
		{
			if (_doc.SelectedIndex != -1)
			{
				Post currpost = _doc[_doc.SelectedIndex];
				currpost.SetThreadRead (true);
				_doc[_doc.SelectedIndex] = currpost;
			}
		}

		private void markNewButton_Click (object sender, EventArgs e)
		{
			if (_doc.SelectedIndex != -1)
			{
				Post currpost = _doc[_doc.SelectedIndex];
				currpost.SetThreadRead (false);
				currpost.SetParentRead (true);

				_doc[_doc.SelectedIndex] = currpost;
			}
		}

		private void checkUpdateMenuItem_Click (object sender, EventArgs e)
		{
			updater.Start ();
		}

		private void prevComment_Click (object sender, EventArgs e)
		{
			postThree.PrevComment ();
		}

		private void nextComment_Click (object sender, EventArgs e)
		{
			postThree.NextComment ();
		}

		private void postThree_MoveToComment (object sender, PostThree.MoveToCommentEventArgs e)
		{
			prevComment.Enabled = prevCommentMenuItem.Enabled = !e.IsFirstNewComment;
			nextComment.Enabled = nextCommentMenuItem.Enabled = !e.IsLastNewComment;
		}

		private void clearLogMenuItem_Click (object sender, EventArgs e)
		{
			_log.Clear ();
		}

		void UpdateNewOnlyComments ()
		{
			newOnlyButton.Checked = Config.Instance.NewOnlyComments;
			newOnlyMenuItem.Checked = Config.Instance.NewOnlyComments;
		}

		private void newOnlyButton_Click (object sender, EventArgs e)
		{
			Config.Instance.NewOnlyComments = !Config.Instance.NewOnlyComments;
		}
	}
}