using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Net;
using System.IO;
using System.Windows.Forms;
using System.ComponentModel;

using Core.LJElements;
using Core.Log;
using Core;


namespace LJWatcher.Update
{
	public class Updater: Component
	{
		/// <summary>
		///   ,  
		/// </summary>
		Version _currentVersion = new Version("1.0.0");

		public Version CurrentVersion
		{
			get { return _currentVersion; }
			set { _currentVersion = value; }
		}

		/// <summary>
		///   
		/// </summary>
		string _url = "http://jenyay.net/archive/ljwatcher.txt";

		public string Url
		{
			get { return _url; }
			set { _url = value; }
		}

		/// <summary>
		///  
		/// </summary>
		string _programUrl = "http://jenyay.net/Soft/Ljwatcher";

		public string ProgramUrl
		{
			get { return _programUrl; }
			set { _programUrl = value; }
		}

		/// <summary>
		///   
		/// </summary>
		Version _lastVersion;

		/// <summary>
		///   
		/// </summary>
		string _updateData;

		/// <summary>
		///     
		/// </summary>
		string _changeLog;

		private System.ComponentModel.BackgroundWorker backgroundWorker;

		string _template = "templates\\update.htm";

		public string Template
		{
			get { return _template; }
			set { _template = value; }
		}

		public Updater ()
		{
			InitializeComponent ();
		}

		public void Start ()
		{
			backgroundWorker.RunWorkerAsync ();
		}

		string MakeHtml ()
		{
			try
			{
				string templatePath = Path.Combine 
					(Path.GetDirectoryName (Application.ExecutablePath),_template);

				StringBuilder template = new StringBuilder ();

				using (TextReader reader = 
					new StreamReader (templatePath, Encoding.GetEncoding(1251)))
				{
					template.Append (reader.ReadToEnd ());
				}

				template.Replace ("{lastver}", _lastVersion.ToString (3));
				template.Replace ("{currver}", _currentVersion.ToString (3));
				template.Replace ("{updatedate}", _updateData);
				template.Replace ("{changelog}", _changeLog);
				template.Replace ("{url}", _programUrl);

				return template.ToString ();
			}
			catch (Exception)
			{
				return "";
			}
		}

		private string SendRequest (string url)
		{
			WebProxy proxy = null;

			if (Config.Instance.UseProxy)
			{
				string proxyUrl = Config.Instance.ProxyAddress;
				int proxyPort = Config.Instance.ProxyPort;

				if (!proxyUrl.StartsWith ("http://"))
				{
					proxyUrl = "http://" + proxyUrl;
				}

				if (proxyUrl.EndsWith ("/"))
				{
					proxyUrl = proxyUrl.Substring (0, proxyUrl.Length - 1);
				}

				proxy = new WebProxy (string.Format ("{0}:{1}/", proxyUrl, proxyPort));
			}

			string ressource = "";

			HttpWebRequest req = (HttpWebRequest)WebRequest.Create (url);
			req.Proxy = proxy;
			req.Method = "GET";
			req.UserAgent = "User-Agent: ljwatcher";
			req.Accept = "text/html";

			req.CookieContainer = new CookieContainer ();

			using (HttpWebResponse result = (HttpWebResponse)req.GetResponse ())
			{
				using (StreamReader rx =
					new StreamReader (result.GetResponseStream (), Encoding.GetEncoding (1251))) 
				{
					ressource = rx.ReadToEnd ();
				}
			}

			return ressource;
		}

		private void InitializeComponent ()
		{
			this.backgroundWorker = new System.ComponentModel.BackgroundWorker ();
			// 
			// backgroundWorker
			// 
			this.backgroundWorker.DoWork += new System.ComponentModel.DoWorkEventHandler (this.backgroundWorker_DoWork);
			this.backgroundWorker.RunWorkerCompleted += new System.ComponentModel.RunWorkerCompletedEventHandler (this.backgroundWorker_RunWorkerCompleted);
			// 
			// Updater
			// 

		}

		private void backgroundWorker_DoWork (object sender, System.ComponentModel.DoWorkEventArgs e)
		{
			try
			{
				string updateData = SendRequest (_url);
				string[] items = updateData.Split ("\n".ToCharArray ());

				_lastVersion = new Version (items[0].Trim ());
				_updateData = items[1].Trim ();

				StringBuilder sb = new StringBuilder ();
				for (int i = 3; i < items.Length; i++)
				{
					sb.AppendLine (items[i].Trim ());
				}

				_changeLog = sb.ToString ();
			}
			catch (Exception)
			{
			}

		}

		private void backgroundWorker_RunWorkerCompleted (object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
		{
			if (_currentVersion < _lastVersion)
			{
				string html = MakeHtml ();
				UpdateDialog dlg = new UpdateDialog (html);
				//dlg.Show();
				dlg.ShowDialog ();
			}
			else
			{
				MessageBox.Show ("    ", "  ",
					 MessageBoxButtons.OK, MessageBoxIcon.Information);
			}

		}
	}
}
